package fr.eurecom.ratcom.email;

import java.util.ArrayList;
import java.util.Random;

import fr.eurecom.ratcom.Ratcom;
import fr.eurecom.ratcom.Server;
import fr.eurecom.ratcom.multicast.Multicast;
import fr.eurecom.ratcom.properties.ProjectProperties;
import fr.eurecom.ratcom.util.FileHandler;
import fr.eurecom.ratcom.util.Logging;
import fr.eurecom.ratcom.vrsu.RebroadcastAlert;
import fr.eurecom.ratcom.vrsu.VirtualRoadSideThread;
import fr.eurecom.ratcom.xml.XMLAlertParser;

/**
 * Reads e-mail thread, at each defined time reads the e-mail to see if there is
 * anything new
 * 
 * @author camara
 * 
 */
public class ReadEmailDeamon extends Thread {
	private static int alertID = 0; // The ID of the sent alert
	

	private static ProjectProperties properties = ProjectProperties
			.getInstance();
	Multicast multicastServer;
	static Logging logging = Logging.getLogger();// ("smsLog.txt");
	Ratcom server;
	XMLAlertParser xmlParser = new XMLAlertParser();

	/**
	 * Constructor
	 */
	public ReadEmailDeamon() {
		super(properties.emailThreadName);
		multicastServer = new Multicast();
	}

	public ReadEmailDeamon(Multicast multicastServer, Ratcom server) {
		super(properties.emailThreadName);
		this.multicastServer = multicastServer;
		this.server = server;
	}

	/**
	 * starts the thread
	 */
	public void run() {
		while (true) {
			logging.logln("Email: reading e-mail");

			ArrayList<Email> list = null;
			// To read the REAL emails in the server

			if (!properties.debugMode) {
				list = new RetrieveMail().retrieve(properties.username,
						properties.password, 0, properties.removeEmail);

			} else {
				// For TEST only, reads the local test xml files with the
				// messages
				list = new ArrayList<Email>();
				Email emailTest = new Email();
				emailTest.setBody(forTest());
				list.add(emailTest);
			}

			if (list.size() != 0) {
				int nEmails = list.size();
				logging.logln("Email: received " + nEmails);
				for (int i = 0; i < nEmails; i++) {
					Email email = list.get(i);
					String message = email.getBody().trim();

					if (message.startsWith("Content-Type")) {
						int cutUntil = message.indexOf("<?xml");
						if (cutUntil < 0) {
							message = "";
						} else
							message = message.substring(cutUntil);
					}

					String msgToShow = xmlParser.getMessageToShow(message);
					if (msgToShow.length() > 0) { // recognized the message and
													// has somehting to show
						alertID = xmlParser.lastMsgID;
						message = properties.alertMsgStart
								+ properties.ID_Separator + alertID
								+ properties.ID_Separator + msgToShow;
						multicastServer.serverSend(message);
						if (server != null)
							server.showNewMessage(" >>>> Sending ALERT!!!!	 <<<< \n" + message, alertID);

						RebroadcastAlert vRSU;
						if (properties.startVRSU) {
							vRSU = new RebroadcastAlert(message, alertID,
									properties.vRSU_rebroadcastTimes,
									properties.vRSU_maxInterval);
							vRSU.start();
						}
					}

				}
			} else {
				logging.logln("Email: no email");

			}

			try {
				sleep(properties.sleepingTimeEmailReading * 1000);// in milisecs
			} catch (InterruptedException e) {
				logging.logln("Email: reading e-mail problem -> "
						+ e.getMessage());

				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		}
	}

	String[] testFiles = { properties.xmlTestFile1, properties.xmlTestFile2,
			properties.xmlTestFile3 };

	private String forTest() {
		String returnValue = "";
		returnValue = FileHandler.readFileUTF8(testFiles[new Random()
				.nextInt(2)]);
		return returnValue;

	}

}
