package fr.eurecom.ratcom.email;

import javax.mail.*;
import javax.mail.internet.MimeBodyPart;

import fr.eurecom.ratcom.properties.ProjectProperties;

import java.util.*;
import java.io.*;

/**
 * Ratcom <br>
 * 
 * This class retrieves the e-mails from the email account.
 * 
 * @file email.ReceiveMail.java
 * @author Daniel_2
 * @date 16 oct. 08 - 21:16:11
 */
public class RetrieveMail {

	ProjectProperties properties = ProjectProperties.getInstance();

	/**
	 * Main method
	 * 
	 * @param args
	 * 
	 * @date 16 oct. 08 - 22:17:39 : Daniel_2
	 */
	public static void main(String[] args) {
		ArrayList<Email> list = new RetrieveMail().retrieve(
				"conferences.page@gmail.com", "trolomoq", 0, false);
		System.out.print(list.size());
	}

	/**
	 * 
	 * @param msg
	 * @return
	 * @throws Exception
	 * 
	 * @date 16 oct. 08 - 22:18:12 : Daniel_2
	 */
	private String handleMultipart(Message msg) throws Exception {
		String disposition;
		String returnValue = "";
		BodyPart part;
		if(msg.getContentType().toUpperCase().startsWith("TEXT/PLAIN"))
			return msg.getContent().toString().replaceAll("3D", "");
		
		Multipart mp = (Multipart) msg.getContent();
		int mpCount = mp.getCount();

		// Passes through the multiparts to find
		// the email body
		for (int m = 0; m < mpCount; m++) {
			part = mp.getBodyPart(m);
			disposition = part.getDisposition();

			if (disposition != null) {
				if (disposition.toLowerCase().equals(
						Part.ATTACHMENT.toLowerCase()))
					System.out.print("Well there is an "
							+ "attach, and now?!?\n");
				else if (disposition.toLowerCase().equals(
						Part.INLINE.toLowerCase())) {
					if (returnValue.trim().length() == 0
							|| !returnValue.contains("text/plain"))
						returnValue = part.getContent().toString().replaceAll("3D", "");
				}
			} else {
				OutputStream output = new OutputStream() {
					private StringBuilder string = new StringBuilder();

					@Override
					public void write(int b) throws IOException {
						this.string.append((char) b);
					}

					public String toString() {
						return this.string.toString();
					}
				};
				// Check if plain
				MimeBodyPart mbp = (MimeBodyPart) part;
				if (mbp != null
						&& (returnValue.trim().length() == 0 || !returnValue
								.contains("text/plain")
						// !returnValue.contains("text/html")
						)) {
					mbp.writeTo(output);
					returnValue = output.toString().replaceAll("3D", "");
				}
			}
		}
		return returnValue;
	}


	/**
	 * Remove an specific e-mail from the Server
	 * 
	 * @param username
	 * @param password
	 * @param mail
	 *            Index - The index of the message to remove
	 * 
	 * @return - true if OK false otherwise
	 */
	public boolean remove(String username, String password, int mailIndex) {
		// Connection
		Properties props = System.getProperties();
		Session session = Session.getInstance(props, null);
		// session.setDebug(true);

		Store store;
		try {
			store = session.getStore("imaps");
			store.connect(properties.host, properties.port, username, password);
			Folder inbox = store.getFolder(properties.folder);
			inbox.open(Folder.READ_WRITE);
			BufferedReader reader = new BufferedReader(new InputStreamReader(
					System.in));
			// Get directory
			Message message[] = inbox.getMessages();
			message[mailIndex].setFlag(Flags.Flag.DELETED, true);

			// Close connection
			inbox.close(true);
			store.close();

		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return true;
	}

	/**
	 * Retrieve the emails from the properties configured Server
	 * 
	 * @param username
	 * @param password
	 * @param startMailIndex
	 *            - Start index from where the messages should be considered
	 * 
	 * @return - Array with the e-mails in an Email class format
	 * 
	 * @date 16 oct. 08 - 22:19:25 : Daniel_2
	 */
	public ArrayList<Email> retrieve(String username, String password,
			int startMailIndex, boolean remove) {
		ArrayList<Email> messages = new ArrayList<Email>(10);

		try {

			// Connection
			Properties props = System.getProperties();
			Session session = Session.getInstance(props, null);
			// session.setDebug(true);

			Store store = session.getStore("imaps");
			store.connect(properties.host, properties.port, username, password);
			Folder inbox = store.getFolder(properties.folder);
			if(remove)
				inbox.open(Folder.READ_WRITE);
			else
				inbox.open(Folder.READ_ONLY);

			grabMessages(startMailIndex, messages, inbox, remove);

			// Close connection
			if (inbox != null) {
				inbox.close(true);
			}

			if (store != null) {
				store.close();
			}
			return messages;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return messages;
	}

	/**
	 * @param startMailIndex - From where should start to read the folder
	 * @param messages Stores the messages
	 * @param inbox - Folder to read
	 * @param remove - if should remove the e-mail from the folder
	 * @throws MessagingException
	 * @throws Exception
	 * 
	 * @date 17 oct. 08 - 23:27:58 : Daniel_2
	 */
	private void grabMessages(int startMailIndex, ArrayList<Email> messages,
			Folder inbox, boolean remove) throws MessagingException, Exception {
		int nMessages = inbox.getMessageCount();

		// if there is no new message, returns without do anything
		if (nMessages < startMailIndex)
			return;

		// Retrieve messages
		Message[] msg = inbox.getMessages();

		BufferedReader reader = new BufferedReader(new InputStreamReader(
				System.in, "UTF-8"));

		// Read the e-mails
		for (int i = startMailIndex; i < msg.length; i++) {
			Email email = new Email();
			email.setSender(msg[i].getFrom()[0].toString());
			email.setSubject(msg[i].getSubject());
			email.setReceivedData(msg[i].getReceivedDate().toString());
			email.setBody(handleMultipart(msg[i]));
			messages.add(email);
			if(remove)
				msg[i].setFlag(Flags.Flag.DELETED, true);

		}
	}
}
