package fr.eurecom.ratcom.vrsu;

import java.net.SocketException;
import java.util.HashSet;

import fr.eurecom.ratcom.ClientDeamon;
import fr.eurecom.ratcom.Ratcom;
import fr.eurecom.ratcom.multicast.Multicast;
import fr.eurecom.ratcom.properties.ProjectProperties;
import fr.eurecom.ratcom.util.Logging;
import fr.eurecom.ratcom.util.StringHandler;

public class VirtualRoadSideThread extends Thread {
	private static ProjectProperties properties = ProjectProperties
			.getInstance();

	private VirtualRoadSideUnitNet vRSU = new VirtualRoadSideUnitNet();

	private HashSet<Integer> broadcastedIDS = new HashSet<Integer>();

	Ratcom ratcomClient;

	Logging logging = Logging.getLogger();// ("smsLog.txt");

	/**
	 * Constructor
	 * 
	 * @param ratcomClient
	 */
	public VirtualRoadSideThread(Ratcom ratcomClient) {
		super(properties.vrsuThreadName);
		this.ratcomClient = ratcomClient;
	}

	/**
	 * starts the thread
	 */
	public void run() {
		while (true) {
			String alertMessage;
			try {
				alertMessage = vRSU.receiveMessage();
				alertMessage = StringHandler.removeEmpty(alertMessage);
				logging.logln("vRSU Thread Received msg " + alertMessage);

				String[] vRSU_alert = StringHandler.splitedString(alertMessage,
						properties.ID_Separator);
				int id = 0;
				if (vRSU_alert.length > 2) {
					id = Integer.parseInt(vRSU_alert[1]);
				}
				// handle alert, show it!!!
				ratcomClient.showNewMessage(" >>>> Received via Virtual Roadside Unit (vRSU) <<<< \n" + alertMessage, id);

				// rebroadcast the alert
				String bodyMsg = alertMessage.substring(vRSU_alert[0].length()
						+ vRSU_alert[1].length() + 2);
				RebroadcastAlert rebroadcast = rebroadcastMessage(id, bodyMsg);
			} catch (SocketException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				logging.logln("vRSU problems with the socket -> "
						+ e.getMessage());
				break;
			}
		}
	}

	/**
	 * Ask for broadcast a new message through the vRSU mechanism
	 * 
	 * @param id
	 * @param message
	 * @return
	 */
	public RebroadcastAlert rebroadcastMessage(int id, String message) {
		if (!broadcastedIDS.contains(id)) {
			broadcastedIDS.add(id);
			RebroadcastAlert rebroadcast = new RebroadcastAlert(message, id,
					properties.vRSU_rebroadcastTimes,
					properties.vRSU_maxInterval);
			rebroadcast.start();
			return rebroadcast;
		} else
			return null;
	}

}
